/* wptCommonCtl.h - Win32 Imagelist headerfile
 *	Copyright (C) 2004-2006, 2009 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */

#ifndef WPT_COMMONCTL_H
#define WPT_COMMONCTL_H

#include <commctrl.h>

/* Symbol constants for the toolbar. */
enum km_sym_tid_t {
    IMI_KEY_NEW=0,  
    IMI_KEY_DELETE,
    IMI_KEY_PROPS,
    IMI_KEY_SIGN,
    IMI_KEY_SEARCH,
    IMI_KEY_FILE_IMPORT,
    IMI_KEY_FILE_EXPORT,
    IMI_KEY_IMPORT,
    IMI_KEY_EXPORT,    
    IMI_SORT_UPARROW,
    IMI_SORT_DOWNARROW,
    IMI_EXIT,
    IMAGELIST_NUMIMAGES
};

/*-- List View --*/
#define KEYLIST_SORT_DESC 0x8000

typedef int (CALLBACK *listview_cmp) (LPARAM lParam1, LPARAM lParam2,
				      LPARAM lParamSort);

/* Listview column. */
struct listview_column_s {
    int pos;		    /* position of the column. */
    int width;		    /* width of the column. */
    char *fieldname;	    /* name of the column. */
};
typedef struct listview_column_s * listview_column_t;

/* Listview control context. */
struct listview_ctrl_s {
    HWND ctrl;		/* the actual listview window. */
    int cols;		/* number of columns. */
    int items;		/* number of items. */
    unsigned int ext_chkbox:1;	/* exteneded style: checkboxes. */
    HIMAGELIST hil;	/* Imagelist context. */
};
typedef struct listview_ctrl_s * listview_ctrl_t;


void listview_new (listview_ctrl_t *ctx, HWND ctrl);
void listview_release (listview_ctrl_t ctx);
int listview_add_column (listview_ctrl_t ctx, listview_column_t col);
int  listview_add_item_pos (listview_ctrl_t ctx, int pos);
int listview_add_item (listview_ctrl_t ctx, const char *text);
int listview_add_item2 (listview_ctrl_t ctx, const char * text, void * magic);
void listview_add_sub_item (listview_ctrl_t ctx, int pos, int col, 
			    const char *text);
int listview_count_items (listview_ctrl_t ctx, int curr_sel);
int listview_del_item (listview_ctrl_t ctx, int pos);
int listview_del_sel_items (listview_ctrl_t ctx);
int listview_del_all_items (listview_ctrl_t ctx);
int listview_get_item_state (listview_ctrl_t ctx, int pos);
int listview_get_selected_item (listview_ctrl_t lv);
void* listview_get_item2 (listview_ctrl_t ctx, int pos);
int listview_set_item2 (listview_ctrl_t ctx, int pos, void *magic);
int listview_sort_items (listview_ctrl_t ctx, int sortby,
			 listview_cmp sort_cb);
int listview_get_curr_pos (listview_ctrl_t ctx);
int listview_get_item_text (listview_ctrl_t ctx, int entry, int pos, 
			     char *text, int maxbytes);
void listview_set_ext_style (listview_ctrl_t ctx);
void listview_set_grid_style (listview_ctrl_t ctx);
void listview_set_chkbox_style (listview_ctrl_t ctx);
int listview_set_column_order (listview_ctrl_t ctx, int *array);
int listview_set_column_width (listview_ctrl_t ctx, int col, int width);
int listview_del_items (listview_ctrl_t ctx);
void listview_select_all (listview_ctrl_t ctx);
void listview_deselect_all (listview_ctrl_t ctx);
void listview_select_one (listview_ctrl_t ctx, int pos);
int listview_find (listview_ctrl_t ctx, const char * str, int substr);
void listview_scroll (listview_ctrl_t ctx, int oldpos, int newpos);
int listview_del_column (listview_ctrl_t ctx, int pos);
int listview_add_item_image (listview_ctrl_t ctx, const char *text, int image);
void listview_set_image_list (listview_ctrl_t ctx, int cx, int cy, 
			      HICON *ico, DWORD nicons);

/*-- Image List --*/
int km_imagelist_load (HWND hwnd, HIMAGELIST *r_il);
int km_imagelist_destroy (HIMAGELIST il);
int km_imagelist_getindex (HIMAGELIST il, int icon);
#endif /*WPT_COMMONCTL_H*/
