/* wptErrors.h - Errors headerfile
 *	Copyright (C) 2000-2006, 2009 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */

#ifndef WPT_ERRORS_H
#define WPT_ERRORS_H

#define w32_errno ((int)GetLastError ())

#define DEBUG(MSG)   debug   (MSG, __FILE__, __LINE__)
#define DEBUG_F(MSG) debug_f (MSG, __FILE__, __LINE__)
	
/* Error constants. */
enum winpt_err_t {
    WPTERR_SUCESS         = 0,
    WPTERR_GENERAL        = 1,
    WPTERR_FILE_OPEN      = 2,
    WPTERR_FILE_CREAT     = 3,
    WPTERR_FILE_READ      = 4,
    WPTERR_FILE_WRITE     = 5,
    WPTERR_FILE_CLOSE     = 6,
    WPTERR_FILE_EXIST     = 7,
    WPTERR_FILE_REMOVE    = 8,
    WPTERR_CLIP_OPEN      = 10,
    WPTERR_CLIP_CLOSE     = 11,
    WPTERR_CLIP_EMPTY     = 12,
    WPTERR_CLIP_SET	  = 13,
    WPTERR_CLIP_GET       = 14,
    WPTERR_CLIP_ISEMPTY   = 15,
    WPTERR_CLIP           = 16,	
    WPTERR_CLIP_SECURED   = 17,
    WPTERR_WINSOCK_INIT   = 18,
    WPTERR_WINSOCK_RESOLVE= 19,
    WPTERR_WINSOCK_SOCKET = 20,
    WPTERR_WINSOCK_CONNECT= 21,
    WPTERR_WINSOCK_SENDKEY= 22,
    WPTERR_WINSOCK_RECVKEY= 23,
    WPTERR_WINSOCK_TIMEOUT= 50,
    WPTERR_WINSOCK_PROXY  = 51, /* general error for proxy problems. */
    WPTERR_KEYSERVER_NOTFOUND= 24,
    WPTERR_REGISTRY       = 26,	
    WPTERR_HOTKEY	  = 28,
    WPTERR_DIR_OPEN	  = 33,
    WPTERR_DIR_CREAT      = 34,
    WPTERR_CURR_WND	  = 35,
    WPTERR_CONFIG_FILE    = 36,
    WPTERR_NODATA	  = 37,
    WPTERR_GPG_EXEFILE    = 38,	
    WPTERR_GPG_KEYRINGS   = 39,
    WPTERR_GPG_OPT_KEYRINGS= 40,
    WPTERR_NOCARD	  = 100,
    WPTERR_NOREADER	  = 101,
    WPTERR_NOPGPCARD	  = 102,
};

void debug_end (void);
const char * winpt_strerror (int errid);
void winpt_errmsg (const char * name, int is_file);
void debug_box (const char * msg, const char * file, int line);
void debug_f (const char * msg, const char * file, int line);
void log_debug (const char * format, ...);
int printf_box (const char * title, int style, const char * format, ...);
#define log_box printf_box

/*-- wptTrayPop.cpp --*/
int msg_box (HWND hwnd, const char * text, const char * title, int mode);
int show_msg (HWND hWnd, int millis, LPCTSTR string);
#define status_box(hwnd, text, title) msg_box (hwnd, text, title, MB_OK)

void balloon_msg_disable (void);
void show_balloon_msg (HWND hparwnd, LPCTSTR string, LPCTSTR icon_name);
HWND show_balloon_msg_pos (HWND hparwnd, int millis, int x, int y,
			   LPCTSTR string, LPCTSTR icon_name);

#endif /* WPT_ERRORS_H */
