/* wptImagelist.cpp - Key Manager Imagelist
 *	Copyright (C) 2004, 2006, 2009 Timo Schulz
 *	Copyright (C) 2003 Andreas Jobs
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <commctrl.h>

#include "wptTypes.h"
#include "wptErrors.h"
#include "wptNLS.h"
#include "wptW32API.h"
#include "resource.h"
#include "wptCommonCtl.h"
#include "wptVersion.h"


int kl_il_map[IMAGELIST_NUMIMAGES+2];

#define replace_icon(il_ctx, resid) \
    ImageList_ReplaceIcon ((il_ctx), -1, \
			   LoadIcon (glob_hinst, MAKEINTRESOURCE (resid)))

/* Load the image list with the default icons.
   Return value: 0 on success.  */
int
km_imagelist_load (HWND hwnd, HIMAGELIST *r_il)
{
    HIMAGELIST il = ImageList_Create (16, 16, ILC_COLOR|ILC_MASK,
				      IMAGELIST_NUMIMAGES, 0);
    if (!il) {
        msg_box (hwnd, "Could not create imagelist.", _("Key Manager"), MB_ERR);
	return -1;
    }

    kl_il_map[IMI_KEY_NEW]	       = replace_icon (il, IDI_KEY_NEW);
    kl_il_map[IMI_KEY_DELETE]     = replace_icon (il, IDI_KEY_DELETE);
    kl_il_map[IMI_KEY_PROPS]      = replace_icon (il, IDI_KEY_PROPS);
    kl_il_map[IMI_KEY_SIGN]       = replace_icon (il, IDI_KEY_SIGN);
    kl_il_map[IMI_KEY_SEARCH]     = replace_icon (il, IDI_KEY_SEARCH);
    kl_il_map[IMI_KEY_FILE_IMPORT]= replace_icon (il, IDI_KEY_FILE_IMPORT);
    kl_il_map[IMI_KEY_FILE_EXPORT]= replace_icon (il, IDI_KEY_FILE_EXPORT);
    kl_il_map[IMI_KEY_IMPORT]     = replace_icon (il, IDI_KEY_IMPORT);
    kl_il_map[IMI_KEY_EXPORT]     = replace_icon (il, IDI_KEY_EXPORT);    
    kl_il_map[IMI_SORT_UPARROW]   = replace_icon (il, IDI_SORT_UPARROW);
    kl_il_map[IMI_SORT_DOWNARROW] = replace_icon (il, IDI_SORT_DOWNARROW);
    *r_il = il;
    return 0;
}


/* Free image list. */
int
km_imagelist_destroy (HIMAGELIST il)
{
    int ret = 0;
    if (!ImageList_Destroy (il))
	ret = WPTERR_GENERAL;
    return ret;
}


/* Map an index to an icon. */
int
km_imagelist_getindex (HIMAGELIST il, int icon)
{
    if ((icon < 0) || (icon >= IMAGELIST_NUMIMAGES))
        return -1;
    return kl_il_map[icon];
}
