/* wptKeyEditOwnertrustDlg.cpp - Key onwertrust dialog
 *	Copyright (C) 2000-2006, 2009 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>

#include "resource.h"
#include "wptErrors.h"
#include "wptW32API.h"
#include "wptVersion.h"
#include "wptTypes.h"
#include "wptNLS.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptContext.h" /* for passphrase_s */
#include "wptDlgs.h"
#include "wptKeyEdit.h"


/* Dialog box procedure to set the ownertrust. */
BOOL CALLBACK
keyedit_ownertrust_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static winpt_key_t k;
    gpgme_error_t err;
    gpgme_key_t key=NULL;
    GpgKeyEdit ek;
    HWND lb;
    int index, id = 0;
    
    switch (msg) {
    case WM_INITDIALOG:
        k = (winpt_key_t)lparam;
	if (!k)
	    BUG (0);
        SetWindowText (dlg, _("Change Ownertrust"));
        lb = GetDlgItem (dlg, IDC_KEYTRUST_TRUST);
        listbox_add_string (lb, _("Don't know") );
        listbox_add_string (lb, _("I do NOT trust"));
        listbox_add_string (lb, _("I trust marginally"));
        listbox_add_string (lb, _("I trust fully"));
	listbox_add_string (lb, _("I trust ultimately"));
	SetDlgItemText (dlg, IDC_KEYTRUST_INF,
			_("Please decide how far you trust this user to "
			  "correctly verify other users' keys (by looking "
			  "at passports, checking fingerprint from "
			  "different sources...)?"));
	SetDlgItemText (dlg, IDCANCEL, _("&Cancel"));
	center_window (dlg, NULL);
        SetForegroundWindow (dlg);
        return TRUE;
        
    case WM_DESTROY:
	balloon_msg_disable ();
	break;
	
    case WM_COMMAND:
        switch (LOWORD (wparam)) {
        case IDCANCEL:
            EndDialog (dlg, FALSE);
            return TRUE;
            
        case IDOK:		
            lb = GetDlgItem (dlg, IDC_KEYTRUST_TRUST);
            index = listbox_get_cursel (lb) + 1;
            if (index < 1 || index > 5) {
		show_balloon_msg (GetDlgItem (dlg, IDC_KEYTRUST_TRUST),
				  _("Please choose one entry."), IDI_ERROR);
                return TRUE;
            }
	    /* check if there is a corresponding secret key for the public key. */
	    if (index == 5 && get_seckey (k->keyid, &key)) {
                id = msg_box (dlg, _("Do you really want to set this key to ultimate trust?"),
                              _("Ownertrust"), MB_YESNO);
                if (id == IDNO)
                    return TRUE;
            }
	    k->callback.new_val = index;
	    ek.setKeyID (k->keyid);
	    err = ek.setTrust ((gpgme_validity_t)index);
            if (err)
                msg_box (dlg, gpgme_strerror (err), _("Ownertrust"), MB_ERR);
	    else
		EndDialog (dlg, TRUE);
            return TRUE;
        }
        break;
    }
    return FALSE;
}
