/* wptKeygenDlg.cpp - Key callback generation dialog
 *	Copyright (C) 2001, 2002, 2003, 2005 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>

#include "resource.h"
#include "wptTypes.h"
#include "wptW32API.h"
#include "wptVersion.h"
#include "wptNLS.h"
#include "wptErrors.h"

/* Handle to the progress dialog. */
static HWND dlg_hwnd = NULL;


/* Dialog box param for the keygen progress. */
static BOOL CALLBACK
keygen_cb_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    switch (msg) {
    case WM_INITDIALOG:
        SetDlgItemText (dlg, IDC_KEYPROG_CB, "");
        SetForegroundWindow (dlg);
	SetWindowText (dlg, _("Key Generation - Progress Dialog"));
	center_window (dlg, NULL);
        dlg_hwnd = dlg;
        return TRUE;

    case WM_DESTROY:
	dlg_hwnd = NULL;
	break;
    }
    
    return FALSE;
}


/* Thread to start the progress dialog (async). */
static DWORD WINAPI
keygen_cb_thread (void *opaque)
{
    DialogBoxParam (glob_hinst, (LPCSTR)IDD_WINPT_KEYPROG, 
		    GetActiveWindow (), keygen_cb_dlg_proc, 0);
    ExitThread (0);
    return 0;
}


/* Create the progress thread. */
int
keygen_cb_dlg_create (void)
{
    DWORD tid;
    HANDLE thread_hd;
    SECURITY_ATTRIBUTES sec_attr;
    
    memset (&sec_attr, 0, sizeof sec_attr);
    sec_attr.bInheritHandle = FALSE;
    sec_attr.lpSecurityDescriptor = NULL;
    sec_attr.nLength = sizeof sec_attr;
    thread_hd = CreateThread (&sec_attr, 0, keygen_cb_thread, NULL, 0, &tid);
    if (!thread_hd) {
        msg_box (NULL, "Could not create key generation thread", 
		 _("Key Generation"), MB_ERR);
        return WPTERR_GENERAL;
    }	
    
    return 0;
}


/* GPGME progress callback function. */
void
keygen_cb (void *opaque, const char *what, int type, int curretn, int total)
{
    static char cb[8192] = {0};
    static int cb_pos = 0;
    
    if (!opaque && !what && !type && !curretn && !total) {
        memset (cb, 0, sizeof (cb));
        cb_pos = 0;
        return;
    }
    
    if (cb_pos >= (DIM (cb)-1)) {
	SetDlgItemText (dlg_hwnd, IDC_KEYPROG_CB, "");
        cb_pos = 0;
    }
    cb[cb_pos] = (char)type;
    cb_pos++;
    cb[cb_pos] = '\0';
    
    SetDlgItemText (dlg_hwnd, IDC_KEYPROG_CB, cb);
    if (GetForegroundWindow () != dlg_hwnd)
	SetForegroundWindow (dlg_hwnd);
}


/* Destroy the progress dialog and cleanup.
   If flush is 1 reset the callback state. */
void
keygen_cb_dlg_destroy (int flush)
{
    if (dlg_hwnd) {
        EndDialog (dlg_hwnd, TRUE);
        dlg_hwnd = NULL;
    }
    if (flush)
	keygen_cb (NULL, NULL, 0, 0, 0);
}

