/* wptTextInputDlg.cpp - Dialog for getting text
 *	Copyright (C) 2001-2007 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>

#include "resource.h"
#include "wptTypes.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptContext.h"
#include "wptDlgs.h"
#include "wptW32API.h"
#include "wptNLS.h"
#include "wptErrors.h"

#define MAX_FILESIZE 8388608 /* 8MB */

void
load_file_contents (HWND dlg, const char * file)
{
    size_t size = 0;
    char *buf = NULL;
    FILE *fp;
    
    size = get_file_size (file);
    if (!size)
        return;
    if (size > MAX_FILESIZE) {
	msg_box (dlg, _("Data is too large for copying."), _("Info"), MB_ERR);
	return;
    }
    buf = new char[size+1];
    if (!buf)
        BUG (0);
    memset (buf, 0, size+1);
    fp = fopen (file, "rb");
    if (fp) {
        fread (buf, 1, size, fp);
        fclose (fp);
        SetDlgItemText (dlg, IDC_TEXT_INPUT, buf);
    }
    free_if_alloc (buf);
}


BOOL CALLBACK
text_input_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static text_input_s *ctx;
    const char *file;
    char *p;
    DWORD n;
    
    switch (msg) {
    case WM_INITDIALOG:
        ctx = (text_input_s *)lparam;
	if (!ctx)
	    BUG (0);
        SetWindowText (dlg, _("Text Input"));
        switch (ctx->type)  {
        case 0: 
	    SetDlgItemText (dlg, IDC_TEXT_INFO, 
			    _("Enter the text that was signed"));
	    break;
        }
	SetDlgItemText (dlg, IDCANCEL, _("&Cancel"));
	SetDlgItemText (dlg, IDC_TEXT_FILE, _("&Load"));
        SetForegroundWindow (dlg);
        return TRUE;
        
    case WM_COMMAND:
        switch (LOWORD (wparam)) {
        case IDC_TEXT_FILE:
            file = get_fileopen_dlg (dlg, _("Text Input from File"), 
				     NULL, NULL);
            if (file)
                load_file_contents (dlg, file);
            break;
            
        case IDOK:
	    n = item_get_text_length (dlg, IDC_TEXT_INPUT);
	    if (n < 1) {
		ctx->length = 0;
		ctx->data = NULL;
	    }
	    else {
		p = new char[n+2];
		if (!p)
		    BUG (0);
		n = GetDlgItemText (dlg, IDC_TEXT_INPUT, p, n+1);
                ctx->length = n;
                ctx->data = p;
            }
            EndDialog (dlg, TRUE);
            return TRUE;
            
        case IDCANCEL:
            ctx->length = 0;
            ctx->data = NULL;
            EndDialog (dlg, FALSE);
            return FALSE;
        }
        break;
    }
    
    return FALSE;
}
